@@ -1,4 +1,5 @@
-"""Trackmania.io API Wrapper
+"""
+Trackmania.io API Wrapper
 ~~~~~~~~~~~~~~~~~~~~~~~~~
 
 A wrapper for Trackmania.io api service.
@@ -17,6 +18,7 @@ from .errors import *
 from .matchmaking import *
 from .player import *
 from .tmmap import *
+from .tmx import *
 from .totd import *
 from .trophy import *
 
@@ -24,7 +26,7 @@ __title__ = "py-tmio"
 __author__ = "Deepesh Nimma"
 __license__ = "MIT"
 __copyright__ = "Copyright 2022-present Deepesh Nimma"
-__version__ = "v0.3.3"
+__version__ = "v0.4.0"
 
 __path__ = __import__("pkgutil").extend_path(__path__, __name__)
 
