@@ -2,10 +2,10 @@ import json
 import logging
 from contextlib import suppress
 from datetime import datetime
-from types import NoneType
 from typing import Dict, List
 
 import redis
+from typing_extensions import Self
 
 from .api import _APIClient
 from .config import Client
@@ -28,7 +28,7 @@ class PlayerMetaInfo:
     """
     .. versionadded :: 0.1.0
 
-    Represents Player Meta Data, which inclues YT, Twitch, Twitter or _TMIO Vanity Link
+    Represents Player Meta Data, which inclues YT, Twitch, Twitter or TMIO Vanity Link
 
     Parameters
     ----------
@@ -39,7 +39,7 @@ class PlayerMetaInfo:
     in_tmgl : bool
         Whether the player is in TMGL
     in_tmio_dev_team : bool
-        Whether the player is in _TMIO Dev Team
+        Whether the player is in TMIO Dev Team
     is_sponsor : bool
         Whether the player is a sponsor
     sponsor_level : int | None
@@ -51,7 +51,7 @@ class PlayerMetaInfo:
     youtube : str | None
         The YouTube URL of the player, `NoneType` if the player has no YouTube
     vanity : str | None
-        The _TMIO Vanity URL of the player, `NoneType` if the player has no _TMIO Vanity URL
+        The TMIO Vanity URL of the player, `NoneType` if the player has no TMIO Vanity URL
     """
 
     def __init__(
@@ -100,20 +100,16 @@ class PlayerMetaInfo:
         _log.debug(f"Creating a PlayerMetaInfo class from the given dictionary.")
 
         return cls(
-            display_url=meta_data["display_url"]
-            if "display_url" in meta_data
-            else None,
-            in_nadeo=meta_data["nadeo"] if "nadeo" in meta_data else False,
-            in_tmgl=meta_data["tmgl"] if "tmgl" in meta_data else False,
-            in_tmio_dev_team=meta_data["team"] if "team" in meta_data else False,
-            is_sponsor=meta_data["sponsor"] if "sponsor" in meta_data else False,
-            sponsor_level=meta_data["sponsor_level"]
-            if "sponsor_level" in meta_data
-            else 0,
-            twitch=meta_data["twitch"] if "twitch" in meta_data else None,
-            twitter=meta_data["twitter"] if "twitter" in meta_data else None,
-            youtube=meta_data["youtube"] if "youtube" in meta_data else None,
-            vanity=meta_data["vanity"] if "vanity" in meta_data else None,
+            display_url=meta_data.get("displayurl"),
+            in_nadeo=meta_data.get("nadeo", False),
+            in_tmgl=meta_data.get("tmgl", False),
+            in_tmio_dev_team=meta_data.get("team", False),
+            is_sponsor=meta_data.get("sponsor", False),
+            sponsor_level=meta_data.get("sponsor_level", 0),
+            twitch=meta_data.get("twitch"),
+            twitter=meta_data.get("twitter"),
+            youtube=meta_data.get("youtube"),
+            vanity=meta_data.get("vanity"),
         )
 
 
@@ -140,7 +136,7 @@ class PlayerZone:
         self.rank = rank
 
     @classmethod
-    def _parse_zones(cls, zones: Dict, zone_positions: List[int]) -> List:
+    def _parse_zones(cls: Self, zones: Dict, zone_positions: List[int]) -> List[Self]:
         """
           .. versionadded :: 0.1.0
 
@@ -175,6 +171,53 @@ class PlayerZone:
 
         return player_zone_list
 
+    @staticmethod
+    def to_string(
+        player_zones: List[Self] | None, add_pos: bool = True, inline: bool = False
+    ) -> str | None:
+        """
+        .. versionadded :: 0.4.0
+
+        Prints a list of zones in a readable format.
+
+        Parameters
+        ----------
+        player_zones : :class:`List[Self]`
+            The list of :class:`PlayerZone` objects.
+        add_pos : bool
+            .. versionadded:: 0.4.0
+            Whether to add the position of the zone.
+        inline : bool
+            .. versionadded :: 0.4.0
+            Whether to print the zones in a single line.
+
+        Returns
+        -------
+        str
+            The list of zones in a readable format.
+        """
+        if player_zones is None:
+            return None
+
+        zone_str = ""
+
+        if not inline:
+            if add_pos:
+                for zone in player_zones:
+                    zone_str = zone_str + zone.zone + " - " + str(zone.rank) + "\n"
+            else:
+                for zone in player_zones:
+                    zone_str = zone_str + zone.zone + "\n"
+        else:
+            if add_pos:
+                zone_str = ", ".join(
+                    f"{zone.zone} - {zone.rank}" for zone in player_zones
+                )
+            else:
+                zone_str = ", ".join(zone.zone for zone in player_zones)
+
+        return zone_str
+
 
 class PlayerSearchResult:
     """
@@ -215,23 +258,19 @@ class PlayerSearchResult:
         self.royal = royal
 
     @classmethod
-    def _from_dict(cls, player_data: Dict):
+    def _from_dict(cls: Self, player_data: Dict) -> Self:
         _log.debug("Creating a PlayerSearchResult class from given dictionary")
 
         zone = (
             PlayerZone._parse_zones(player_data["player"]["zone"], [0, 0, 0, 0, 0])
-            if "zone" in player_data
+            if "zone" in player_data["player"]
             else None
         )
-        club_tag = (
-            player_data["player"]["club_tag"]
-            if "club_tag" in player_data["player"]
-            else None
-        )
-        name = player_data["player"]["name"]
-        player_id = player_data["player"]["id"]
+        club_tag = player_data.get("player").get("club_tag", None)
+        name = player_data.get("player").get("name")
+        player_id = player_data.get("player").get("id")
         matchmaking = PlayerMatchmaking._from_dict(
-            player_data["matchmaking"], player_id
+            player_data.get("matchmaking"), player_id
         )
 
         return cls(club_tag, name, player_id, zone, matchmaking[0], matchmaking[1])
@@ -296,7 +335,7 @@ class Player:
 
     def __str__(self):
         """String representation of the class."""
-        return f"Player: {self.name} ({self.name})"
+        return f"Player: {self.name} ({self.player_id})"
 
     @property
     def first_login(self):
@@ -309,7 +348,7 @@ class Player:
         return self._id
 
     @classmethod
-    async def get(cls, player_id: str):
+    async def get_player(cls: Self, player_id: str) -> Self:
         """
         .. versionadded :: 0.1.0
 
@@ -353,9 +392,11 @@ class Player:
     @staticmethod
     async def search(
         username: str,
-    ) -> None | PlayerSearchResult | List[PlayerSearchResult]:
+    ) -> List[PlayerSearchResult] | None:
         """
         .. versionadded :: 0.1.0
+        .. versionchanged :: 0.3.4
+            The function no longer returns a single :class:`PlayerSearchResult`. It will now always return a `list` or `None`
 
         Searches for a player's information using their username.
 
@@ -366,8 +407,9 @@ class Player:
 
         Returns
         -------
-        :class:`NoneType` | :class:`PlayerSearchResult` | :class:`List[PlayerSearchResult]`
-            None if no players. :class:`PlayerSearchResult` if only one player. :class:`List[PlayerSearchResult]` if multiple players.
+        :class:`List[PlayerSearchResult]` | None
+            Returns a list of :class:`PlayerSearchResult` with users who have similar usernames. Returns `None`
+            if no user with that username can be found.
         """
         _log.debug(f"Searching for players with the username -> {username}")
 
@@ -378,24 +420,23 @@ class Player:
         await api_client.close()
 
         with suppress(KeyError, TypeError):
-
             raise TMIOException(search_result["error"])
 
         if len(search_result) == 0:
             return None
-        elif len(search_result) == 1:
-            return PlayerSearchResult._from_dict(search_result[0])
-        else:
-            players = []
-            for player in search_result:
-                players.append(PlayerSearchResult._from_dict(player))
 
-            return players
+        players = []
+        for player in search_result:
+            players.append(PlayerSearchResult._from_dict(player))
+
+        return players
 
     @staticmethod
     async def get_id(username: str) -> str:
         """
         .. versionadded :: 0.1.0
+        .. versionadded :: 0.3.4
+            Updated to work with the change in `search` function
 
         Gets a player's id from the given username
 
@@ -420,16 +461,10 @@ class Player:
 
         players = await Player.search(username)
 
-        if isinstance(players, PlayerSearchResult):
-            with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
-                _log.debug(f"Caching {username.lower()} id as {players.player_id}")
-                cache_client.set(f"{username.lower()}:id", players.player_id)
-            return players.player_id
-        elif isinstance(players, list):
-            with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
-                _log.debug(f"Caching {username.lower()} id as {players[0].player_id}")
-                cache_client.set(f"{username.lower()}:id", players[0].player_id)
-            return players[0].player_id
+        with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
+            _log.debug(f"Caching {username.lower()} id as {players[0].player_id}")
+            cache_client.set(f"{username.lower()}:id", players[0].player_id)
+        return players[0].player_id
 
     @staticmethod
     async def get_username(player_id: str) -> str:
@@ -458,7 +493,7 @@ class Player:
                     cache_client.get(f"{player_id}:username").decode("utf-8")
                 )
 
-        player: Player = await Player.get(player_id)
+        player: Player = await Player.get_player(player_id)
 
         with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
             _log.debug(f"Caching {player_id}:username as {player.name}")
@@ -470,6 +505,8 @@ class Player:
     def _parse_player(player_data: Dict) -> Dict:
         """
         .. versionadded :: 0.1.0
+        .. versionchanged :: 0.4.0
+            Optimized everything!
 
         Parses the player data
 
@@ -483,12 +520,14 @@ class Player:
         :class:`Dict`
             The parsed player data formatted kwargs friendly for the :class:`Player` constructors
         """
+        # Parsing First Login
         first_login = (
             datetime.strptime(player_data["timestamp"], "%Y-%m-%dT%H:%M:%S+00:00")
             if "timestamp" in player_data
             else None
         )
 
+        # Parsing Last Club Tag Change
         last_club_tag_change = (
             datetime.strptime(
                 player_data["clubtagtimestamp"], "%Y-%m-%dT%H:%M:%S+00:00"
@@ -497,75 +536,52 @@ class Player:
             else None
         )
 
-        # player_meta = (
-        #     PlayerMetaInfo._from_dict(player_data["meta"])
-        #     if "meta" in player_data
-        #     else PlayerMetaInfo._from_dict(dict())
-        # )
+        # Parsing Meta
+        if player_data.get("meta") is not None:
+            if isinstance(player_data["meta"], PlayerMetaInfo):
+                player_meta = player_data.get("meta")
+            else:
+                player_meta = PlayerMetaInfo._from_dict(player_data.get("meta"))
+        else:
+            player_meta = PlayerMetaInfo._from_dict(dict())
 
-        player_meta = (
-            player_data["meta"]
-            if "meta" in player_data and isinstance(player_data["meta"], PlayerMetaInfo)
-            else PlayerMetaInfo._from_dict(player_data["meta"])
-            if "meta" in player_data
-            else PlayerMetaInfo._from_dict(dict())
-        )
-
-        player_trophies = (
-            PlayerTrophies._from_dict(
-                player_data["trophies"],
-                player_data["accountid"]
-                if "accountid" in player_data
-                else player_data["player_id"]
-                if "player_id" in player_data
-                else None,
+        # Parsing Trophies
+        player_trophies = player_data.get("trophies")
+        if player_trophies is not None:
+            player_trophies = PlayerTrophies._from_dict(
+                player_trophies,
+                player_data.get("accountid", player_data.get("player_id")),
             )
-            if "trophies" in player_data
-            and not isinstance(player_data["trophies"], NoneType)
-            else None
-        )
-        player_zone = (
-            PlayerZone._parse_zones(
-                player_data["trophies"]["zone"],
-                player_data["trophies"]["zonepositions"],
+
+        # Parsing Zones
+        if (
+            player_trophies is not None
+            and player_data.get("trophies").get("zone") is not None
+        ):
+            player_zone = PlayerZone._parse_zones(
+                player_data.get("trophies").get("zone"),
+                player_data.get("trophies").get("zonepositions"),
             )
-            if "trophies" in player_data
-            and not isinstance(player_data["trophies"], NoneType)
-            and "zone" in player_data["trophies"]
-            and not isinstance(player_data["trophies"]["zone"], NoneType)
-            else None
-        )
+        else:
+            player_zone = False
 
-        player_id = (
-            player_data["accountid"]
-            if "accountid" in player_data
-            else player_data["id"]
-            if "id" in player_data
-            else player_data["playerid"]
-            if "playerid" in player_data
-            else None
+        # Parsing player id
+        player_id = player_data.get(
+            "accountid", player_data.get("id", player_data.get("playerid", None))
         )
 
+        # Parsing Matchmaking
         matchmaking = (
             PlayerMatchmaking._from_dict(player_data["matchmaking"], player_id)
             if "matchmaking" in player_data
             else [None, None]
         )
 
-        club_tag = (
-            player_data["clubtag"]
-            if "clubtag" in player_data
-            else player_data["tag"]
-            if "tag" in player_data
-            else None
-        )
-        name = (
-            player_data["displayname"]
-            if "displayname" in player_data
-            else player_data["name"]
-            if "name" in player_data
-            else None
-        )
+        # Parsing Club Tag
+        club_tag = player_data.get("clubtag", player_data.get("tag", None))
+
+        # Parsing Name
+        name = player_data.get("displayname", player_data.get("name", None))
 
         return {
             "club_tag": club_tag,
