@@ -6,7 +6,9 @@ from types import NoneType
 from typing import Dict, List
 
 import redis
+from typing_extensions import Self
 
+from ._util import _add_commas
 from .api import _APIClient
 from .config import Client
 from .constants import _TMIO
@@ -14,7 +16,76 @@ from .errors import InvalidIDError, InvalidTrophyNumber, TMIOException
 
 _log = logging.getLogger(__name__)
 
-__all__ = ("PlayerTrophies",)
+__all__ = ("PlayerTrophies", "TrophyLeaderboardPlayer")
+
+
+class TrophyLeaderboardPlayer:
+    """
+    .. versionadded :: 0.4.0
+
+    Represents a player on the trophy leaderboards
+
+    Parameters
+    ----------
+    player_name : str
+        The player's name
+    club_tag : str | None
+        The player's club tag
+    player_id : str
+        The player's ID
+    rank : int
+        The player's rank
+    score : str
+        The player's score
+    zones : `List[PlayerZone]`
+        The player's zones
+    """
+
+    def __init__(
+        self,
+        player_name: str,
+        club_tag: str | None,
+        player_id: str,
+        rank: int,
+        score: str,
+        zones: List,
+    ):
+        self.player_name = player_name
+        self.club_tag = club_tag
+        self.player_id = player_id
+        self.rank = rank
+        self.score = score
+        self.zones = zones
+
+    @classmethod
+    def _from_dict(cls: Self, raw: Dict) -> Self:
+        from .player import PlayerZone
+
+        args = []
+        player = raw.get("player")
+        args.append(player.get("name"))
+        args.append(player.get("tag", None))
+        args.append(player.get("id"))
+        args.append(raw.get("rank"))
+        args.append(_add_commas(int(raw.get("score"))))
+        args.append(PlayerZone._parse_zones(player.get("zone"), [0, 0, 0, 0, 0]))
+
+        return cls(*args)
+
+    async def get_player(self: Self):
+        """
+        .. versionadded :: 0.4.0
+
+        Gets the player object using the player ID.
+
+        Returns
+        -------
+        :class:`Player`
+            The player object
+        """
+        from .player import Player
+
+        return await Player.get_player(self.player_id)
 
 
 class PlayerTrophies:
@@ -53,7 +124,7 @@ class PlayerTrophies:
         self._player_id = player_id
 
     @classmethod
-    def _from_dict(cls, raw_trophy_data: Dict, player_id: str):
+    def _from_dict(cls: Self, raw_trophy_data: Dict, player_id: str) -> Self:
         """
         Creates a :class:`PlayerTrophies` object from the given dictionary.
 
@@ -71,12 +142,12 @@ class PlayerTrophies:
         _log.debug("Creating a PlayerTrophies class from the given dictionary.")
 
         return cls(
-            echelon=raw_trophy_data["echelon"],
+            echelon=raw_trophy_data.get("echelon"),
             last_change=datetime.strptime(
-                raw_trophy_data["timestamp"], "%Y-%m-%dT%H:%M:%S+00:00"
+                raw_trophy_data.get("timestamp"), "%Y-%m-%dT%H:%M:%S+00:00"
             ),
-            points=raw_trophy_data["points"],
-            trophies=raw_trophy_data["counts"],
+            points=raw_trophy_data.get("points"),
+            trophies=raw_trophy_data.get("counts"),
             player_id=player_id,
         )
 
@@ -119,7 +190,16 @@ class PlayerTrophies:
         return self.trophies[number - 1]
 
     def score(self) -> int:
-        """Returns the trophy score of the player."""
+        """
+        .. versionadded :: 0.3.0
+
+        Returns the total trophy score of the player.
+
+        Returns
+        -------
+        int
+            The total score.
+        """
 
         score = (
             0
@@ -137,6 +217,15 @@ class PlayerTrophies:
 
         return score
 
+    def __str__(self) -> str:
+        trophy_str = ""
+        for i, trophyd in enumerate(self.trophies):
+            trophy_str = trophy_str + f"T{i + 1} - " + _add_commas(trophyd) + "\n"
+
+        trophy_str = trophy_str + f"\nTotal Trophies: {_add_commas(self.score())}"
+
+        return trophy_str
+
     async def history(self, page: int = 0) -> Dict:
         """
         .. versionadded :: 0.3.0
@@ -193,7 +282,7 @@ class PlayerTrophies:
         return history["gains"]
 
     @staticmethod
-    async def top(page: int = 0) -> Dict:
+    async def top(page: int = 0) -> List[TrophyLeaderboardPlayer]:
         """
         .. versionadded :: 0.3.0
 
@@ -206,8 +295,8 @@ class PlayerTrophies:
 
         Returns
         -------
-        Dict
-            The players
+        :class:`List[TrophyLeaderboardPlayer]`
+            The players as a list of :class:`TrophyLeaderboardPlayer` objects.
         """
         _log.debug(f"Getting Page {page} of Trophy Leaderboards")
 
@@ -216,9 +305,13 @@ class PlayerTrophies:
         with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
             if cache_client.exists(f"trophies:{page}"):
                 _log.debug(f"Found trophy leaderboard for page {page} in cache")
-                return json.loads(cache_client.get(f"trophies:{page}").decode("utf-8"))[
-                    "ranks"
-                ]
+                all_players = json.loads(
+                    cache_client.get(f"trophies:{page}").decode("utf-8")
+                )
+
+                lb_players = []
+                for top_player in all_players["ranks"]:
+                    lb_players.append(TrophyLeaderboardPlayer._from_dict(top_player))
 
         api_client = _APIClient()
 
@@ -234,4 +327,8 @@ class PlayerTrophies:
             _log.debug(f"Caching trophy leaderboard for page: {page}")
             cache_client.set(f"trophies:{page}", json.dumps(top_trophies), ex=3600)
 
-        return top_trophies["ranks"]
+        lb_players = []
+        for top_player in top_trophies["ranks"]:
+            lb_players.append(TrophyLeaderboardPlayer._from_dict(top_player))
+
+        return lb_players
